@@ -1,13 +1,17 @@
+# mypy: allow-untyped-defs, allow-incomplete-defs, allow-untyped-calls
+# mypy: no-warn-return-any, allow-any-generics
+
 from __future__ import annotations
 
+import re
 from typing import Any
+from typing import Dict
 from typing import List
 from typing import Optional
 from typing import TYPE_CHECKING
 from typing import Union
 
 from sqlalchemy import types as sqltypes
-from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.schema import Column
 from sqlalchemy.schema import CreateIndex
 from sqlalchemy.sql.base import Executable
@@ -28,6 +32,7 @@ from .base import RenameTable
 from .impl import DefaultImpl
 from .. import util
 from ..util import sqla_compat
+from ..util.sqla_compat import compiles
 
 if TYPE_CHECKING:
     from typing import Literal
@@ -35,7 +40,6 @@ if TYPE_CHECKING:
     from sqlalchemy.dialects.mssql.base import MSDDLCompiler
     from sqlalchemy.dialects.mssql.base import MSSQLCompiler
     from sqlalchemy.engine.cursor import CursorResult
-    from sqlalchemy.engine.cursor import LegacyCursorResult
     from sqlalchemy.sql.schema import Index
     from sqlalchemy.sql.schema import Table
     from sqlalchemy.sql.selectable import TableClause
@@ -50,28 +54,23 @@ class MSSQLImpl(DefaultImpl):
     batch_separator = "GO"
 
     type_synonyms = DefaultImpl.type_synonyms + ({"VARCHAR", "NVARCHAR"},)
-    identity_attrs_ignore = (
+    identity_attrs_ignore = DefaultImpl.identity_attrs_ignore + (
         "minvalue",
         "maxvalue",
         "nominvalue",
         "nomaxvalue",
         "cycle",
         "cache",
-        "order",
-        "on_null",
-        "order",
     )
 
     def __init__(self, *arg, **kw) -> None:
-        super(MSSQLImpl, self).__init__(*arg, **kw)
+        super().__init__(*arg, **kw)
         self.batch_separator = self.context_opts.get(
             "mssql_batch_separator", self.batch_separator
         )
 
-    def _exec(
-        self, construct: Any, *args, **kw
-    ) -> Optional[Union["LegacyCursorResult", "CursorResult"]]:
-        result = super(MSSQLImpl, self)._exec(construct, *args, **kw)
+    def _exec(self, construct: Any, *args, **kw) -> Optional[CursorResult]:
+        result = super()._exec(construct, *args, **kw)
         if self.as_sql and self.batch_separator:
             self.static_output(self.batch_separator)
         return result
@@ -80,27 +79,27 @@ class MSSQLImpl(DefaultImpl):
         self.static_output("BEGIN TRANSACTION" + self.command_terminator)
 
     def emit_commit(self) -> None:
-        super(MSSQLImpl, self).emit_commit()
+        super().emit_commit()
         if self.as_sql and self.batch_separator:
             self.static_output(self.batch_separator)
 
-    def alter_column(  # type:ignore[override]
+    def alter_column(
         self,
         table_name: str,
         column_name: str,
+        *,
         nullable: Optional[bool] = None,
         server_default: Optional[
-            Union["_ServerDefault", "Literal[False]"]
+            Union[_ServerDefault, Literal[False]]
         ] = False,
         name: Optional[str] = None,
-        type_: Optional["TypeEngine"] = None,
+        type_: Optional[TypeEngine] = None,
         schema: Optional[str] = None,
-        existing_type: Optional["TypeEngine"] = None,
-        existing_server_default: Optional["_ServerDefault"] = None,
+        existing_type: Optional[TypeEngine] = None,
+        existing_server_default: Optional[_ServerDefault] = None,
         existing_nullable: Optional[bool] = None,
         **kw: Any,
     ) -> None:
-
         if nullable is not None:
             if type_ is not None:
                 # the NULL/NOT NULL alter will handle
@@ -139,7 +138,7 @@ class MSSQLImpl(DefaultImpl):
             kw["server_default"] = server_default
             kw["existing_server_default"] = existing_server_default
 
-        super(MSSQLImpl, self).alter_column(
+        super().alter_column(
             table_name,
             column_name,
             nullable=nullable,
@@ -161,7 +160,7 @@ class MSSQLImpl(DefaultImpl):
                     )
                 )
             if server_default is not None:
-                super(MSSQLImpl, self).alter_column(
+                super().alter_column(
                     table_name,
                     column_name,
                     schema=schema,
@@ -169,11 +168,11 @@ class MSSQLImpl(DefaultImpl):
                 )
 
         if name is not None:
-            super(MSSQLImpl, self).alter_column(
+            super().alter_column(
                 table_name, column_name, schema=schema, name=name
             )
 
-    def create_index(self, index: "Index") -> None:
+    def create_index(self, index: Index, **kw: Any) -> None:
         # this likely defaults to None if not present, so get()
         # should normally not return the default value.  being
         # defensive in any case
@@ -182,28 +181,29 @@ class MSSQLImpl(DefaultImpl):
         for col in mssql_include:
             if col not in index.table.c:
                 index.table.append_column(Column(col, sqltypes.NullType))
-        self._exec(CreateIndex(index))
+        self._exec(CreateIndex(index, **kw))
 
     def bulk_insert(  # type:ignore[override]
-        self, table: Union["TableClause", "Table"], rows: List[dict], **kw: Any
+        self, table: Union[TableClause, Table], rows: List[dict], **kw: Any
     ) -> None:
         if self.as_sql:
             self._exec(
                 "SET IDENTITY_INSERT %s ON"
                 % self.dialect.identifier_preparer.format_table(table)
             )
-            super(MSSQLImpl, self).bulk_insert(table, rows, **kw)
+            super().bulk_insert(table, rows, **kw)
             self._exec(
                 "SET IDENTITY_INSERT %s OFF"
                 % self.dialect.identifier_preparer.format_table(table)
             )
         else:
-            super(MSSQLImpl, self).bulk_insert(table, rows, **kw)
+            super().bulk_insert(table, rows, **kw)
 
     def drop_column(
         self,
         table_name: str,
-        column: "Column",
+        column: Column[Any],
+        *,
         schema: Optional[str] = None,
         **kw,
     ) -> None:
@@ -224,9 +224,7 @@ class MSSQLImpl(DefaultImpl):
         drop_fks = kw.pop("mssql_drop_foreign_key", False)
         if drop_fks:
             self._exec(_ExecDropFKConstraint(table_name, column, schema))
-        super(MSSQLImpl, self).drop_column(
-            table_name, column, schema=schema, **kw
-        )
+        super().drop_column(table_name, column, schema=schema, **kw)
 
     def compare_server_default(
         self,
@@ -235,21 +233,25 @@ class MSSQLImpl(DefaultImpl):
         rendered_metadata_default,
         rendered_inspector_default,
     ):
-        def clean(value):
-            if value is not None:
-                value = value.strip()
-                while value[0] == "(" and value[-1] == ")":
-                    value = value[1:-1]
-            return value
+        if rendered_metadata_default is not None:
+            rendered_metadata_default = re.sub(
+                r"[\(\) \"\']", "", rendered_metadata_default
+            )
 
-        return clean(rendered_inspector_default) != clean(
-            rendered_metadata_default
-        )
+        if rendered_inspector_default is not None:
+            # SQL Server collapses whitespace and adds arbitrary parenthesis
+            # within expressions.   our only option is collapse all of it
+
+            rendered_inspector_default = re.sub(
+                r"[\(\) \"\']", "", rendered_inspector_default
+            )
+
+        return rendered_inspector_default != rendered_metadata_default
 
     def _compare_identity_default(self, metadata_identity, inspector_identity):
-        diff, ignored, is_alter = super(
-            MSSQLImpl, self
-        )._compare_identity_default(metadata_identity, inspector_identity)
+        diff, ignored, is_alter = super()._compare_identity_default(
+            metadata_identity, inspector_identity
+        )
 
         if (
             metadata_identity is None
@@ -264,6 +266,17 @@ class MSSQLImpl(DefaultImpl):
 
         return diff, ignored, is_alter
 
+    def adjust_reflected_dialect_options(
+        self, reflected_object: Dict[str, Any], kind: str
+    ) -> Dict[str, Any]:
+        options: Dict[str, Any]
+        options = reflected_object.get("dialect_options", {}).copy()
+        if not options.get("mssql_include"):
+            options.pop("mssql_include", None)
+        if not options.get("mssql_clustered"):
+            options.pop("mssql_clustered", None)
+        return options
+
 
 class _ExecDropConstraint(Executable, ClauseElement):
     inherit_cache = False
@@ -271,7 +284,7 @@ class _ExecDropConstraint(Executable, ClauseElement):
     def __init__(
         self,
         tname: str,
-        colname: Union["Column", str],
+        colname: Union[Column[Any], str],
         type_: str,
         schema: Optional[str],
     ) -> None:
@@ -285,7 +298,7 @@ class _ExecDropFKConstraint(Executable, ClauseElement):
     inherit_cache = False
 
     def __init__(
-        self, tname: str, colname: "Column", schema: Optional[str]
+        self, tname: str, colname: Column[Any], schema: Optional[str]
     ) -> None:
         self.tname = tname
         self.colname = colname
@@ -294,7 +307,7 @@ class _ExecDropFKConstraint(Executable, ClauseElement):
 
 @compiles(_ExecDropConstraint, "mssql")
 def _exec_drop_col_constraint(
-    element: "_ExecDropConstraint", compiler: "MSSQLCompiler", **kw
+    element: _ExecDropConstraint, compiler: MSSQLCompiler, **kw
 ) -> str:
     schema, tname, colname, type_ = (
         element.schema,
@@ -304,9 +317,8 @@ def _exec_drop_col_constraint(
     )
     # from http://www.mssqltips.com/sqlservertip/1425/\
     # working-with-default-constraints-in-sql-server/
-    # TODO: needs table formatting, etc.
     return """declare @const_name varchar(256)
-select @const_name = [name] from %(type)s
+select @const_name = QUOTENAME([name]) from %(type)s
 where parent_object_id = object_id('%(schema_dot)s%(tname)s')
 and col_name(parent_object_id, parent_column_id) = '%(colname)s'
 exec('alter table %(tname_quoted)s drop constraint ' + @const_name)""" % {
@@ -320,12 +332,12 @@ exec('alter table %(tname_quoted)s drop constraint ' + @const_name)""" % {
 
 @compiles(_ExecDropFKConstraint, "mssql")
 def _exec_drop_col_fk_constraint(
-    element: "_ExecDropFKConstraint", compiler: "MSSQLCompiler", **kw
+    element: _ExecDropFKConstraint, compiler: MSSQLCompiler, **kw
 ) -> str:
     schema, tname, colname = element.schema, element.tname, element.colname
 
     return """declare @const_name varchar(256)
-select @const_name = [name] from
+select @const_name = QUOTENAME([name]) from
 sys.foreign_keys fk join sys.foreign_key_columns fkc
 on fk.object_id=fkc.constraint_object_id
 where fkc.parent_object_id = object_id('%(schema_dot)s%(tname)s')
@@ -339,34 +351,34 @@ exec('alter table %(tname_quoted)s drop constraint ' + @const_name)""" % {
 
 
 @compiles(AddColumn, "mssql")
-def visit_add_column(
-    element: "AddColumn", compiler: "MSDDLCompiler", **kw
-) -> str:
+def visit_add_column(element: AddColumn, compiler: MSDDLCompiler, **kw) -> str:
     return "%s %s" % (
         alter_table(compiler, element.table_name, element.schema),
         mssql_add_column(compiler, element.column, **kw),
     )
 
 
-def mssql_add_column(compiler: "MSDDLCompiler", column: "Column", **kw) -> str:
+def mssql_add_column(
+    compiler: MSDDLCompiler, column: Column[Any], **kw
+) -> str:
     return "ADD %s" % compiler.get_column_specification(column, **kw)
 
 
 @compiles(ColumnNullable, "mssql")
 def visit_column_nullable(
-    element: "ColumnNullable", compiler: "MSDDLCompiler", **kw
+    element: ColumnNullable, compiler: MSDDLCompiler, **kw
 ) -> str:
     return "%s %s %s %s" % (
         alter_table(compiler, element.table_name, element.schema),
         alter_column(compiler, element.column_name),
-        format_type(compiler, element.existing_type),
+        format_type(compiler, element.existing_type),  # type: ignore[arg-type]
         "NULL" if element.nullable else "NOT NULL",
     )
 
 
 @compiles(ColumnDefault, "mssql")
 def visit_column_default(
-    element: "ColumnDefault", compiler: "MSDDLCompiler", **kw
+    element: ColumnDefault, compiler: MSDDLCompiler, **kw
 ) -> str:
     # TODO: there can also be a named constraint
     # with ADD CONSTRAINT here
@@ -379,7 +391,7 @@ def visit_column_default(
 
 @compiles(ColumnName, "mssql")
 def visit_rename_column(
-    element: "ColumnName", compiler: "MSDDLCompiler", **kw
+    element: ColumnName, compiler: MSDDLCompiler, **kw
 ) -> str:
     return "EXEC sp_rename '%s.%s', %s, 'COLUMN'" % (
         format_table_name(compiler, element.table_name, element.schema),
@@ -390,7 +402,7 @@ def visit_rename_column(
 
 @compiles(ColumnType, "mssql")
 def visit_column_type(
-    element: "ColumnType", compiler: "MSDDLCompiler", **kw
+    element: ColumnType, compiler: MSDDLCompiler, **kw
 ) -> str:
     return "%s %s %s" % (
         alter_table(compiler, element.table_name, element.schema),
@@ -401,7 +413,7 @@ def visit_column_type(
 
 @compiles(RenameTable, "mssql")
 def visit_rename_table(
-    element: "RenameTable", compiler: "MSDDLCompiler", **kw
+    element: RenameTable, compiler: MSDDLCompiler, **kw
 ) -> str:
     return "EXEC sp_rename '%s', %s" % (
         format_table_name(compiler, element.table_name, element.schema),
