@@ -642,9 +642,11 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestBase):
         diffs = {
             (
                 cmd,
-                isinstance(obj, (UniqueConstraint, Index))
-                if obj.name is not None
-                else False,
+                (
+                    isinstance(obj, (UniqueConstraint, Index))
+                    if obj.name is not None
+                    else False
+                ),
             )
             for cmd, obj in diffs
         }
@@ -1230,7 +1232,6 @@ class AutogenerateIndexTest(AutogenFixtureTest, TestBase):
         eq_(diffs, [])
 
     @config.requirements.covering_indexes
-    @config.requirements.sqlalchemy_14
     def test_nothing_changed_covering_index(self):
         m1 = MetaData()
         m2 = MetaData()
@@ -1800,7 +1801,6 @@ class NoUqReflectionIndexTest(NoUqReflection, AutogenerateUniqueIndexTest):
 
 
 class NoUqReportsIndAsUqTest(NoUqReflectionIndexTest):
-
     """this test suite simulates the condition where:
 
     a. the dialect doesn't report unique constraints
