@@ -1,15 +1,19 @@
 from sqlalchemy import Boolean
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import DATETIME
 from sqlalchemy import exc
 from sqlalchemy import Float
 from sqlalchemy import func
+from sqlalchemy import Identity
 from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
+from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import text
 from sqlalchemy import TIMESTAMP
+from sqlalchemy.dialects.mysql import VARCHAR
 
 from alembic import op
 from alembic import util
@@ -25,7 +29,6 @@ from alembic.testing.env import staging_env
 from alembic.testing.fixtures import AlterColRoundTripFixture
 from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TestBase
-from alembic.util import sqla_compat
 
 
 class MySQLOpTest(TestBase):
@@ -63,6 +66,16 @@ class MySQLOpTest(TestBase):
             "ALTER TABLE t ADD COLUMN q INTEGER COMMENT 'This is a comment'"
         )
 
+    def test_add_column_if_not_exists(self):
+        context = op_fixture("mysql")
+        op.add_column("t", Column("c", Integer), if_not_exists=True)
+        context.assert_("ALTER TABLE t ADD COLUMN IF NOT EXISTS c INTEGER")
+
+    def test_drop_column_if_exists(self):
+        context = op_fixture("mysql")
+        op.drop_column("t", "c", if_exists=True)
+        context.assert_("ALTER TABLE t DROP COLUMN IF EXISTS c")
+
     def test_rename_column(self):
         context = op_fixture("mysql")
         op.alter_column(
@@ -229,6 +242,22 @@ class MySQLOpTest(TestBase):
         op.invoke(operation)
         context.assert_("ALTER TABLE t MODIFY c FLOAT NULL DEFAULT 0")
 
+    def test_alter_column_change_collation(self):
+        context = op_fixture("mysql")
+        op.alter_column(
+            "t1",
+            "c1",
+            nullable=False,
+            existing_type=String(50),
+            type_=VARCHAR(
+                50, charset="utf8mb4", collation="utf8mb4/utf8mb4_unicode_ci"
+            ),
+        )
+        context.assert_(
+            "ALTER TABLE t1 MODIFY c1 VARCHAR(50) "
+            "CHARACTER SET utf8mb4 COLLATE utf8mb4/utf8mb4_unicode_ci NOT NULL"
+        )
+
     def test_col_not_nullable(self):
         context = op_fixture("mysql")
         op.alter_column("t1", "c1", nullable=False, existing_type=Integer)
@@ -382,12 +411,11 @@ class MySQLOpTest(TestBase):
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("ALTER TABLE foo.t2 COMMENT ''")
 
-    @config.requirements.computed_columns_api
     def test_add_column_computed(self):
         context = op_fixture("mysql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+            Column("some_column", Integer, Computed("foo * 5")),
         )
         context.assert_(
             "ALTER TABLE t1 ADD COLUMN some_column "
@@ -395,13 +423,19 @@ class MySQLOpTest(TestBase):
         )
 
     def test_drop_fk(self):
+        context = op_fixture("mysql")
+        op.drop_constraint("f1", "t1", type_="foreignkey")
+        context.assert_("ALTER TABLE t1 DROP FOREIGN KEY f1")
+
+    def test_drop_fk_legacy(self):
+        """#1245"""
         context = op_fixture("mysql")
         op.drop_constraint("f1", "t1", "foreignkey")
         context.assert_("ALTER TABLE t1 DROP FOREIGN KEY f1")
 
     def test_drop_fk_quoted(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyFk", "MyTable", "foreignkey")
+        op.drop_constraint("MyFk", "MyTable", type_="foreignkey")
         context.assert_("ALTER TABLE `MyTable` DROP FOREIGN KEY `MyFk`")
 
     def test_drop_constraint_primary(self):
@@ -411,32 +445,32 @@ class MySQLOpTest(TestBase):
 
     def test_drop_unique(self):
         context = op_fixture("mysql")
-        op.drop_constraint("f1", "t1", "unique")
+        op.drop_constraint("f1", "t1", type_="unique")
         context.assert_("ALTER TABLE t1 DROP INDEX f1")
 
     def test_drop_unique_quoted(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyUnique", "MyTable", "unique")
+        op.drop_constraint("MyUnique", "MyTable", type_="unique")
         context.assert_("ALTER TABLE `MyTable` DROP INDEX `MyUnique`")
 
     def test_drop_check_mariadb(self):
         context = op_fixture("mariadb")
-        op.drop_constraint("f1", "t1", "check")
+        op.drop_constraint("f1", "t1", type_="check")
         context.assert_("ALTER TABLE t1 DROP CONSTRAINT f1")
 
     def test_drop_check_quoted_mariadb(self):
         context = op_fixture("mariadb")
-        op.drop_constraint("MyCheck", "MyTable", "check")
+        op.drop_constraint("MyCheck", "MyTable", type_="check")
         context.assert_("ALTER TABLE `MyTable` DROP CONSTRAINT `MyCheck`")
 
     def test_drop_check_mysql(self):
         context = op_fixture("mysql")
-        op.drop_constraint("f1", "t1", "check")
+        op.drop_constraint("f1", "t1", type_="check")
         context.assert_("ALTER TABLE t1 DROP CHECK f1")
 
     def test_drop_check_quoted_mysql(self):
         context = op_fixture("mysql")
-        op.drop_constraint("MyCheck", "MyTable", "check")
+        op.drop_constraint("MyCheck", "MyTable", type_="check")
         context.assert_("ALTER TABLE `MyTable` DROP CHECK `MyCheck`")
 
     def test_drop_unknown(self):
@@ -448,7 +482,7 @@ class MySQLOpTest(TestBase):
             op.drop_constraint,
             "f1",
             "t1",
-            "typo",
+            type_="typo",
         )
 
     def test_drop_generic_constraint(self):
@@ -463,14 +497,13 @@ class MySQLOpTest(TestBase):
         )
 
     @combinations(
-        (lambda: sqla_compat.Computed("foo * 5"), lambda: None),
-        (lambda: None, lambda: sqla_compat.Computed("foo * 5")),
+        (lambda: Computed("foo * 5"), lambda: None),
+        (lambda: None, lambda: Computed("foo * 5")),
         (
-            lambda: sqla_compat.Computed("foo * 42"),
-            lambda: sqla_compat.Computed("foo * 5"),
+            lambda: Computed("foo * 42"),
+            lambda: Computed("foo * 5"),
         ),
     )
-    @config.requirements.computed_columns_api
     def test_alter_column_computed_not_supported(self, sd, esd):
         op_fixture("mysql")
         assert_raises_message(
@@ -486,14 +519,10 @@ class MySQLOpTest(TestBase):
         )
 
     @combinations(
-        (lambda: sqla_compat.Identity(), lambda: None),
-        (lambda: None, lambda: sqla_compat.Identity()),
-        (
-            lambda: sqla_compat.Identity(),
-            lambda: sqla_compat.Identity(),
-        ),
+        (lambda: Identity(), lambda: None),
+        (lambda: None, lambda: Identity()),
+        (lambda: Identity(), lambda: Identity()),
     )
-    @config.requirements.identity_columns_api
     def test_alter_column_identity_not_supported(self, sd, esd):
         op_fixture()
         assert_raises_message(
@@ -627,7 +656,7 @@ class MySQLDefaultCompareTest(TestBase):
         insp = inspect(self.bind)
         cols = insp.get_columns(t1.name)
         refl = Table(t1.name, MetaData())
-        sqla_compat._reflect_table(insp, refl, None)
+        insp.reflect_table(refl, include_columns=None)
         ctx = self.autogen_context["context"]
         return ctx.impl.compare_server_default(
             refl.c[cols[0]["name"]], col, rendered, cols[0]["default"]
